#include <sstream>
#include <vector>
#include <iostream>
#include <string>
#include "histogram.h"
#include "text.h"
#include "svg.h"

#include <curl/curl.h>
using namespace std;
struct Input {
    vector<double> numbers;
    size_t bin_count{};
    string stroke;
};

Input
input_data(istream& in, bool prompt) {
    int i;
    size_t number_count;
    string check1 = " ";
    string check2 = "#";
 
    if (prompt) { cerr << "Enter number count: "; };
    in >> number_count;

    Input in1;
    in1.numbers.resize(number_count);
    if (prompt) { cerr << "Enter numbers: "; };
    for (size_t i = 0; i < number_count; i++) {
        in >> in1.numbers[i];
    }

    if (prompt) { cerr << "Enter bin count: "; };
    in >> in1.bin_count;

 
    if (prompt) { cerr << "Enter stroke colour without spaces or in code format:"; };
    in.ignore();
    getline(in, in1.stroke);

    for (i = 0; i < in1.stroke.length(); i++) {
        if (isspace(in1.stroke[i])) {
            cerr << "invalid input";
            exit(0);
        }
    }
    if (in1.stroke[0] == check2[0]) {
        if (in1.stroke.length() != 7) {
            cerr << "invalid input";
            exit(0);
        }
    }
    return in1;
}

static size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}
Input
download(const string& address) {
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if (curl) {
        CURLcode res;
        curl_off_t connect;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res != CURLE_OK) {
            fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
            exit(1);
        }
        if (CURLE_OK == res) {
            res = curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME_T, &connect);
            if (CURLE_OK == res) {
                fprintf(stderr, "connection time: %" CURL_FORMAT_CURL_OFF_T ".%06ld", connect / 1000000, (long)(connect % 1000000));
            }
        }
        curl_easy_cleanup(curl);
    }
    return input_data(buffer, false);
}

int
main(int argc, char* argv[])
{
#include <curl/curl.h>
    curl_global_init(CURL_GLOBAL_ALL);
    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    }
    else {
        input = input_data(cin, true);
    }

    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins,"yellow");
}

